@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2004 Noah Levitt
  * Copyright (c) 2007, 2008 Christian Persch
+ * Copyright (c) 2016 DaeHyun Sung
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
  */
 
 #include <config.h>
@@ -24,24 +25,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#if GTK_CHECK_VERSION (2, 90, 7)
-#define GDK_KEY(symbol) GDK_KEY_##symbol
-#else
-#include <gdk/gdkkeysyms.h>
-#define GDK_KEY(symbol) GDK_##symbol
-#endif
-
-#include "gucharmap-charmap.h"
-#include "gucharmap-unicode-info.h"
+#include "gucharmap.h"
 #include "gucharmap-marshal.h"
 #include "gucharmap-private.h"
 
 struct _GucharmapCharmapPrivate {
-#if !GTK_CHECK_VERSION (2, 15, 0)
-  GtkOrientation orientation;
-  GtkWidgetClass *paned_class;
-#endif
-
   GtkWidget *notebook;
   GucharmapChaptersView *chapters_view;
   GucharmapChartable *chartable;
@@ -69,9 +57,6 @@ enum
 
 enum {
   PROP_0,
-#if !GTK_CHECK_VERSION (2, 15, 0)
-  PROP_ORIENTATION,
-#endif
   PROP_CHAPTERS_MODEL,
   PROP_ACTIVE_CHAPTER,
   PROP_ACTIVE_CHARACTER,
@@ -95,16 +80,12 @@ gucharmap_charmap_finalize (GObject *object)
   GucharmapCharmap *charmap = GUCHARMAP_CHARMAP (object);
   GucharmapCharmapPrivate *priv = charmap->priv;
 
-  gdk_cursor_unref (priv->hand_cursor);
-  gdk_cursor_unref (priv->regular_cursor);
+  g_object_unref (priv->hand_cursor);
+  g_object_unref (priv->regular_cursor);
 
   if (priv->font_desc)
     pango_font_description_free (priv->font_desc);
 
-#if !GTK_CHECK_VERSION (2, 15, 0)
-  g_type_class_unref (priv->paned_class);
-#endif
-
   G_OBJECT_CLASS (gucharmap_charmap_parent_class)->finalize (object);
 }
 
@@ -118,11 +99,6 @@ gucharmap_charmap_get_property (GObject *object,
   GucharmapCharmapPrivate *priv = charmap->priv;
 
   switch (prop_id) {
-#if !GTK_CHECK_VERSION (2, 15, 0)
-    case PROP_ORIENTATION:
-      g_value_set_enum (value, gucharmap_charmap_get_orientation (charmap));
-      break;
-#endif
     case PROP_CHAPTERS_MODEL:
       g_value_set_object (value, gucharmap_charmap_get_chapters_model (charmap));
       break;
@@ -163,11 +139,6 @@ gucharmap_charmap_set_property (GObject *object,
   GucharmapCharmapPrivate *priv = charmap->priv;
 
   switch (prop_id) {
-#if !GTK_CHECK_VERSION (2, 15, 0)
-    case PROP_ORIENTATION:
-      gucharmap_charmap_set_orientation (charmap, g_value_get_enum (value));
-      break;
-#endif
     case PROP_CHAPTERS_MODEL:
       gucharmap_charmap_set_chapters_model (charmap, g_value_get_object (value));
       break;
@@ -197,37 +168,10 @@ gucharmap_charmap_set_property (GObject *object,
   }
 }
 
-#if !GTK_CHECK_VERSION (2, 15, 0)
-
-static void
-gucharmap_charmap_size_request (GtkWidget *widget,
-                                GtkRequisition *requisition)
-{
-  GucharmapCharmap *charmap = GUCHARMAP_CHARMAP (widget);
-  GucharmapCharmapPrivate *priv = charmap->priv;
-
-  priv->paned_class->size_request (widget, requisition);
-}
-
-static void
-gucharmap_charmap_size_allocate (GtkWidget *widget,
-                                 GtkAllocation  *allocation)
-{
-  GucharmapCharmap *charmap = GUCHARMAP_CHARMAP (widget);
-  GucharmapCharmapPrivate *priv = charmap->priv;
-
-  priv->paned_class->size_allocate (widget, allocation);
-}
-
-#endif /* GTK < 2.15.0 */
-
 static void
 gucharmap_charmap_class_init (GucharmapCharmapClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-#if !GTK_CHECK_VERSION (2, 15, 0)
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-#endif
 
   _gucharmap_intl_ensure_initialized ();
 
@@ -235,11 +179,6 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *klass)
   object_class->set_property = gucharmap_charmap_set_property;
   object_class->finalize = gucharmap_charmap_finalize;
 
-#if !GTK_CHECK_VERSION (2, 15, 0)
-  widget_class->size_request = gucharmap_charmap_size_request;
-  widget_class->size_allocate = gucharmap_charmap_size_allocate;
-#endif
-
   gucharmap_charmap_signals[STATUS_MESSAGE] =
       g_signal_new (I_("status-message"), gucharmap_charmap_get_type (),
                     G_SIGNAL_RUN_FIRST,
@@ -254,20 +193,6 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *klass)
                     NULL, NULL, _gucharmap_marshal_VOID__UINT_UINT, G_TYPE_NONE, 
                     2, G_TYPE_UINT, G_TYPE_UINT);
 
-#if !GTK_CHECK_VERSION (2, 15, 0)
-  g_object_class_install_property
-    (object_class,
-     PROP_ORIENTATION,
-     g_param_spec_enum ("orientation", NULL, NULL,
-                        GTK_TYPE_ORIENTATION,
-                        GTK_ORIENTATION_HORIZONTAL,
-                        G_PARAM_READWRITE |
-                        G_PARAM_CONSTRUCT |
-                        G_PARAM_STATIC_NAME |
-                        G_PARAM_STATIC_NICK |
-                        G_PARAM_STATIC_BLURB));
-#endif
-
   g_object_class_install_property
     (object_class,
      PROP_CHAPTERS_MODEL,
@@ -437,8 +362,23 @@ insert_codepoint (GucharmapCharmap *charmap,
 {
   gchar *str;
   GtkTextTag *tag;
+  char buf[7];
+  GUnicodeType t;
+  gboolean is_graph, is_Mn;
+  char nbsp[3] = "\302\240\0"; /* U+00A0 NO-BREAK SPACE */
 
-  str = g_strdup_printf ("U+%4.4X %s", uc,
+  t = g_unichar_type (uc);
+  is_Mn = t == G_UNICODE_NON_SPACING_MARK /* Mn */;
+  is_graph = g_unichar_isgraph (uc);
+
+  buf[g_unichar_to_utf8 (uc, buf)] = '\0';
+
+  str = g_strdup_printf ("%s%s%sU+%4.4X %s",
+                         /* Per unicode standard, exhibit nonspacing marks on NBSP */
+                         is_graph && is_Mn ? nbsp : "",
+                         is_graph ? buf : "",
+                         is_graph ? " " : "",
+                         uc,
                          gucharmap_get_unicode_name (uc));
 
   tag = gtk_text_buffer_create_tag (buffer, NULL, 
@@ -475,6 +415,58 @@ insert_chocolate_detail_codepoints (GucharmapCharmap *charmap,
   gtk_text_buffer_insert (buffer, iter, "\n", -1);
 }
 
+typedef enum {
+  TAG_NONE,
+  TAG_FONT,
+  TAG_NO_BREAK,
+  TAG_INITIAL,
+  TAG_MEDIAL,
+  TAG_FINAL,
+  TAG_ISOLATED,
+  TAG_CIRCLE,
+  TAG_SUPER,
+  TAG_SUB,
+  TAG_VERTICAL,
+  TAG_WIDE,
+  TAG_NARROW,
+  TAG_SMALL,
+  TAG_SQUARE,
+  TAG_FRACTION,
+  TAG_COMPAT
+} CompatibilityFormattingTag;
+
+static void
+insert_compatibility_formatting_tag (GucharmapCharmap *charmap,
+                                     GtkTextBuffer *buffer,
+                                     GtkTextIter *iter,
+                                     CompatibilityFormattingTag tag)
+{
+  static const char *tag_names[] = {
+    [TAG_NONE] = "",
+    [TAG_FONT] = N_("Font variant of"),
+    [TAG_NO_BREAK] = N_("No-break version of"),
+    [TAG_INITIAL] = N_("Initial presentation form of"),
+    [TAG_MEDIAL] = N_("Medial presentation form of"),
+    [TAG_FINAL] = N_("Final presentation form of"),
+    [TAG_ISOLATED] = N_("Isolated presentation form of"),
+    [TAG_CIRCLE] = N_("Encircled form of"),
+    [TAG_SUPER] = N_("Superscript form of"),
+    [TAG_SUB] = N_("Subscript form of"),
+    [TAG_VERTICAL] = N_("Vertical layout presentation form of"),
+    [TAG_WIDE] = N_("Wide compatibility character of"),
+    [TAG_NARROW] = N_("Narrow compatibility character of"),
+    [TAG_SMALL] = N_("Small variant form of"),
+    [TAG_SQUARE] = N_("CJK squared font variant of"),
+    [TAG_FRACTION] = N_("Vulgar fraction form of"),
+    [TAG_COMPAT] = N_("Unspecified compatibility character of")
+  };
+  char *str;
+
+  str = g_strdup_printf ("%s ", _(tag_names[tag]));
+  gtk_text_buffer_insert (buffer, iter, str, -1);
+  g_free (str);
+}
+
 #define is_hex_digit(c) (((c) >= '0' && (c) <= '9') \
                          || ((c) >= 'A' && (c) <= 'F'))
 
@@ -512,6 +504,72 @@ find_codepoint (const gchar *str)
   return NULL;
 }
 
+static CompatibilityFormattingTag
+parse_tag (const char *str,
+           gsize len)
+{
+  if (strncmp (str, "font", len) == 0)
+    return TAG_FONT;
+  if (strncmp (str, "noBreak", len) == 0)
+    return TAG_NO_BREAK;
+  if (strncmp (str, "initial", len) == 0)
+    return TAG_INITIAL;
+  if (strncmp (str, "medial", len) == 0)
+    return TAG_MEDIAL;
+  if (strncmp (str, "final", len) == 0)
+    return TAG_FINAL;
+  if (strncmp (str, "isolated", len) == 0)
+    return TAG_ISOLATED;
+  if (strncmp (str, "circle", len) == 0)
+    return TAG_CIRCLE;
+  if (strncmp (str, "super", len) == 0)
+    return TAG_SUPER;
+  if (strncmp (str, "sub", len) == 0)
+    return TAG_SUB;
+  if (strncmp (str, "vertical", len) == 0)
+    return TAG_VERTICAL;
+  if (strncmp (str, "wide", len) == 0)
+    return TAG_WIDE;
+  if (strncmp (str, "narrow", len) == 0)
+    return TAG_NARROW;
+  if (strncmp (str, "small", len) == 0)
+    return TAG_SMALL;
+  if (strncmp (str, "square", len) == 0)
+    return TAG_SQUARE;
+  if (strncmp (str, "fraction", len) == 0)
+    return TAG_FRACTION;
+  if (strncmp (str, "compat", len) == 0)
+    return TAG_COMPAT;
+
+  g_printerr ("unrecognised tag '%s'\n", str);
+  return TAG_NONE;
+}
+
+/* returns the string after the tag */
+static const char *
+find_tag (const char *str,
+          CompatibilityFormattingTag *tag)
+{
+  const char *csb;
+
+  *tag = TAG_NONE;
+
+  if (str[0] != '<')
+    return str;
+
+  csb = strchr (str, '>');
+  if (csb == NULL)
+    return str;
+
+  *tag = parse_tag (str + 1, csb - str - 1);
+
+  str = csb + 1;
+  while (g_ascii_isspace (*str))
+    str++;
+
+  return str;
+}
+
 static void
 insert_string_link_codepoints (GucharmapCharmap *charmap,
                                GtkTextBuffer *buffer,
@@ -519,8 +577,12 @@ insert_string_link_codepoints (GucharmapCharmap *charmap,
                                const gchar *str)
 {
   const gchar *p1, *p2;
+  CompatibilityFormattingTag tag;
+
+  p1 = find_tag (str, &tag);
+  if (tag != TAG_NONE)
+    insert_compatibility_formatting_tag (charmap, buffer, iter, tag);
 
-  p1 = str;
   for (;;)
     {
       p2 = find_codepoint (p1);
@@ -533,7 +595,9 @@ insert_string_link_codepoints (GucharmapCharmap *charmap,
         }
       else
         {
-          gtk_text_buffer_insert (buffer, iter, p1, -1);
+          /* Ignore non-codepoints for compatibility formatting tags */
+          if (tag == TAG_NONE)
+            gtk_text_buffer_insert (buffer, iter, p1, -1);
           break;
         }
     }
@@ -580,24 +644,43 @@ insert_heading (GucharmapCharmap *charmap,
 }
 
 static void
-conditionally_insert_canonical_decomposition (GucharmapCharmap *charmap, 
-                                              GtkTextBuffer *buffer,
-                                              GtkTextIter *iter,
-                                              gunichar uc)
+conditionally_insert_decomposition (GucharmapCharmap *charmap, 
+                                    GtkTextBuffer *buffer,
+                                    GtkTextIter *iter,
+                                    gunichar uc,
+                                    gboolean compat)
 {
-  gunichar *decomposition;
-  gsize result_len;
-  guint i;
+  gunichar decomposition[G_UNICHAR_MAX_DECOMPOSITION_LENGTH];
+  gsize result_len, i;
 
-  decomposition = g_unicode_canonical_decomposition (uc, &result_len);
+  result_len = g_unichar_fully_decompose (uc,
+                                          compat,
+                                          decomposition,
+                                          G_N_ELEMENTS (decomposition));
 
   if (result_len == 1)
-    {
-      g_free (decomposition);
-      return;
-    }
+    return;
 
-  gtk_text_buffer_insert (buffer, iter, _("Canonical decomposition:"), -1);
+  /* Only insert compat decomp if different from canonical decomp */
+  if (compat) {
+    gunichar canonical[G_UNICHAR_MAX_DECOMPOSITION_LENGTH];
+    gsize canonical_len;
+
+    canonical_len = g_unichar_fully_decompose (uc,
+                                               FALSE /* canonical */,
+                                               canonical,
+                                               G_N_ELEMENTS (canonical));
+
+    if (canonical_len == result_len &&
+        memcmp (decomposition, canonical, result_len * sizeof (gunichar)) == 0)
+      return;
+  }
+
+  gtk_text_buffer_insert (buffer, iter,
+                          compat
+                          ? _("Compatibility decomposition:")
+                          : _("Canonical decomposition:"),
+                          -1);
   gtk_text_buffer_insert (buffer, iter, " ", -1);
 
   insert_codepoint (charmap, buffer, iter, decomposition[0]);
@@ -608,8 +691,6 @@ conditionally_insert_canonical_decomposition (GucharmapCharmap *charmap,
     }
 
   gtk_text_buffer_insert (buffer, iter, "\n", -1);
-
-  g_free (decomposition);
 }
 
 static void
@@ -668,7 +749,9 @@ set_details (GucharmapCharmap *charmap,
                          gucharmap_get_unicode_category_name (uc));
 
   /* canonical decomposition */
-  conditionally_insert_canonical_decomposition (charmap, buffer, &iter, uc);
+  conditionally_insert_decomposition (charmap, buffer, &iter, uc, FALSE);
+  /* compatibility decomposition */
+  conditionally_insert_decomposition (charmap, buffer, &iter, uc, TRUE);
 
   /* representations */
   if (g_unichar_break_type(uc) != G_UNICODE_BREAK_SURROGATE)
@@ -710,7 +793,7 @@ set_details (GucharmapCharmap *charmap,
 	  (0xE000 <= uc && uc <= 0xFFFD) ||
 	  (0x10000 <= uc && uc <= 0x10FFFF))
 	{
-	  temp = g_strdup_printf ("&#%d;", uc);
+	  temp = g_strdup_printf ("&#%u;", uc);
 	  insert_vanilla_detail (charmap, buffer, &iter, 
 				 _("XML decimal entity:"), temp);
 	  g_free (temp);
@@ -771,8 +854,6 @@ set_details (GucharmapCharmap *charmap,
         }
     }
 
-#if ENABLE_UNIHAN
-
   /* this isn't so bad efficiency-wise */
   if (gucharmap_get_unicode_kDefinition (uc)
       || gucharmap_get_unicode_kCantonese (uc)
@@ -780,7 +861,8 @@ set_details (GucharmapCharmap *charmap,
       || gucharmap_get_unicode_kJapaneseOn (uc)
       || gucharmap_get_unicode_kJapaneseKun (uc)
       || gucharmap_get_unicode_kTang (uc)
-      || gucharmap_get_unicode_kKorean (uc))
+      || gucharmap_get_unicode_kHangul(uc)
+      || gucharmap_get_unicode_kVietnamese(uc))
     {
       insert_heading (charmap, buffer, &iter, _("CJK Ideograph Information"));
 
@@ -814,12 +896,16 @@ set_details (GucharmapCharmap *charmap,
         insert_vanilla_detail (charmap, buffer, &iter,
                                _("Tang Pronunciation:"), csp);
     
-      csp = gucharmap_get_unicode_kKorean (uc);
+      csp = gucharmap_get_unicode_kHangul (uc);
       if (csp)
         insert_vanilla_detail (charmap, buffer, &iter,
                                _("Korean Pronunciation:"), csp);
+      
+      csp = gucharmap_get_unicode_kVietnamese (uc);
+      if (csp)
+        insert_vanilla_detail (charmap, buffer, &iter, 
+                               _("Vietnamese Pronunciation:"), csp);
     }
-#endif /* #if ENABLE_UNIHAN */
 }
 
 static void
@@ -854,11 +940,9 @@ chartable_sync_active_char (GtkWidget *widget,
   g_string_append_printf (gs, "U+%4.4X %s", wc, 
                           gucharmap_get_unicode_name (wc));
 
-#if ENABLE_UNIHAN
   temp = gucharmap_get_unicode_kDefinition (wc);
   if (temp)
     g_string_append_printf (gs, "   %s", temp);
-#endif
 
   temps = gucharmap_get_nameslist_equals (wc);
   if (temps)
@@ -964,9 +1048,9 @@ details_key_press_event (GtkWidget *text_view,
 
   switch (event->keyval)
     {
-      case GDK_KEY (Return):
-      case GDK_KEY (ISO_Enter):
-      case GDK_KEY (KP_Enter):
+      case GDK_KEY_Return:
+      case GDK_KEY_ISO_Enter:
+      case GDK_KEY_KP_Enter:
         buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
         gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
                                           gtk_text_buffer_get_insert (buffer));
@@ -1023,12 +1107,9 @@ set_cursor_if_appropriate (GucharmapCharmap *charmap,
 {
   GucharmapCharmapPrivate *priv = charmap->priv;
   GSList *tags = NULL, *tagp = NULL;
-  GtkTextBuffer *buffer;
   GtkTextIter iter;
   gboolean hovering_over_link = FALSE;
 
-  buffer = gtk_text_view_get_buffer (priv->details_view);
-
   gtk_text_view_get_iter_at_location (priv->details_view,
                                       &iter, x, y);
 
@@ -1076,7 +1157,6 @@ details_motion_notify_event (GtkWidget *text_view,
 
   set_cursor_if_appropriate (charmap, x, y);
 
-  gdk_window_get_pointer (gtk_widget_get_window (text_view), NULL, NULL, NULL);
   return FALSE;
 }
 
@@ -1100,11 +1180,7 @@ details_visibility_notify_event (GtkWidget *text_view,
 
 static void
 notebook_switch_page (GtkNotebook *notebook,
-#if GTK_CHECK_VERSION (2, 90, 6)
                       GtkWidget *page,
-#else
-                      gpointer page /* useless */,
-#endif
                       guint page_num,
                       GucharmapCharmap *charmap)
 {
@@ -1174,7 +1250,7 @@ gucharmap_charmap_init (GucharmapCharmap *charmap)
 
   gtk_container_add (GTK_CONTAINER (scrolled_window), view);
   gtk_widget_show (view);
-  gtk_paned_pack1 (GTK_PANED (charmap), scrolled_window, FALSE, TRUE);
+  gtk_paned_pack1 (GTK_PANED (charmap), scrolled_window, FALSE, FALSE);
   gtk_widget_show (scrolled_window);
 
   /* Right pane */
@@ -1183,9 +1259,13 @@ gucharmap_charmap_init (GucharmapCharmap *charmap)
   /* Chartable page */
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+                                  GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
                                        GTK_SHADOW_NONE);
+#if GTK_CHECK_VERSION (3, 15, 9)
+  gtk_scrolled_window_set_overlay_scrolling (GTK_SCROLLED_WINDOW (scrolled_window),
+                                             FALSE);
+#endif
 
   chartable = gucharmap_chartable_new ();
   priv->chartable = GUCHARMAP_CHARTABLE (chartable);
@@ -1220,6 +1300,7 @@ gucharmap_charmap_init (GucharmapCharmap *charmap)
   textview = gtk_text_view_new ();
   priv->details_view = GTK_TEXT_VIEW (textview);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
+  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), FALSE);
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview),
                                GTK_WRAP_WORD);
 
@@ -1261,8 +1342,11 @@ gucharmap_charmap_init (GucharmapCharmap *charmap)
   g_signal_connect (priv->notebook, "switch-page",
                     G_CALLBACK (notebook_switch_page), charmap);
 
-  gtk_paned_pack2 (GTK_PANED (charmap), priv->notebook, TRUE, TRUE);
+  gtk_paned_pack2 (GTK_PANED (charmap), priv->notebook, TRUE, FALSE);
   gtk_widget_show (priv->notebook);
+
+  gtk_widget_set_hexpand (GTK_WIDGET (charmap), TRUE);
+  gtk_widget_set_vexpand (GTK_WIDGET (charmap), TRUE);
 }
 
 GtkWidget *
@@ -1280,43 +1364,13 @@ gucharmap_charmap_new (void)
  * @charmap:
  * @orientation:
  *
- * Deprecated: 2.25.0
+ * Deprecated: 2.26
  */
 void
 gucharmap_charmap_set_orientation (GucharmapCharmap *charmap,
                                    GtkOrientation orientation)
 {
-#if GTK_CHECK_VERSION (2, 15, 0)
   gtk_orientable_set_orientation (GTK_ORIENTABLE (charmap), orientation);
-#else
-  GucharmapCharmapPrivate *priv;
-  GtkPaned *paned = GTK_PANED (charmap);
-
-  g_return_if_fail (GUCHARMAP_IS_CHARMAP (charmap));
-  priv = charmap->priv;
-
-  priv->orientation = orientation;
-
-  if (priv->paned_class)
-    g_type_class_unref (priv->paned_class);
-
-  switch (orientation) {
-    case GTK_ORIENTATION_HORIZONTAL:
-      priv->paned_class = g_type_class_ref (GTK_TYPE_HPANED);
-
-      paned->cursor_type = GDK_SB_H_DOUBLE_ARROW;
-      paned->orientation = GTK_ORIENTATION_VERTICAL;
-      break;
-    case GTK_ORIENTATION_VERTICAL:
-      priv->paned_class = g_type_class_ref (GTK_TYPE_VPANED);
-
-      paned->cursor_type = GDK_SB_V_DOUBLE_ARROW;
-      paned->orientation = GTK_ORIENTATION_HORIZONTAL;
-      break;
-  }
-
-  g_object_notify (G_OBJECT (charmap), "orientation");
-#endif
 }
 
 /**
@@ -1330,11 +1384,7 @@ gucharmap_charmap_get_orientation (GucharmapCharmap *charmap)
 {
   g_return_val_if_fail (GUCHARMAP_IS_CHARMAP (charmap), GTK_ORIENTATION_HORIZONTAL);
 
-#if GTK_CHECK_VERSION (2, 15, 0)
   return gtk_orientable_get_orientation (GTK_ORIENTABLE (charmap));
-#else
-  return charmap->priv->orientation;
-#endif
 }
 
 #endif /* !GUCHARMAP_DISABLE_DEPRECATED_SOURCE */
@@ -1474,6 +1524,12 @@ gucharmap_charmap_previous_chapter (GucharmapCharmap *charmap)
   gucharmap_chapters_view_previous (priv->chapters_view);
 }
 
+/**
+ * gucharmap_charmap_get_chartable:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Returns: (transfer none): a #GucharmapChartable
+ */
 GucharmapChartable *
 gucharmap_charmap_get_chartable (GucharmapCharmap *charmap)
 {
@@ -1510,6 +1566,15 @@ gucharmap_charmap_set_chapters_model (GucharmapCharmap  *charmap,
   g_object_thaw_notify (object);
 }
 
+/**
+ * gucharmap_charmap_get_chapters_model:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Gets the #GucharmapChaptersModel associated with the #GucharmapChaptersView
+ * of @charmap.
+ *
+ * Returns: (transfer none): a #GucharmapChaptersModel
+ */
 GucharmapChaptersModel *
 gucharmap_charmap_get_chapters_model (GucharmapCharmap *charmap)
 {
@@ -1518,14 +1583,29 @@ gucharmap_charmap_get_chapters_model (GucharmapCharmap *charmap)
   return gucharmap_chapters_view_get_model (priv->chapters_view);
 }
 
+/**
+ * gucharmap_charmap_get_chapters_view:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Gets the #GucharmapChaptersView of @charmap
+ *
+ * Returns: (transfer none): the #GucharmapChaptersView
+ */
 GucharmapChaptersView *
-gucharmap_charmap_get_chapters_view  (GucharmapCharmap *charmap)
+gucharmap_charmap_get_chapters_view (GucharmapCharmap *charmap)
 {
   GucharmapCharmapPrivate *priv = charmap->priv;
 
   return priv->chapters_view;
 }
 
+/**
+ * gucharmap_charmap_get_book_codepoint_list:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Returns: (transfer full): the GucharmapCodepointList. Must be freed with
+ * g_object_unref().
+ */
 GucharmapCodepointList *
 gucharmap_charmap_get_book_codepoint_list (GucharmapCharmap *charmap)
 {
@@ -1548,11 +1628,7 @@ gucharmap_charmap_get_chapters_visible (GucharmapCharmap *charmap)
 {
   GucharmapCharmapPrivate *priv = charmap->priv;
 
-#if GTK_CHECK_VERSION (2,18,0)
   return gtk_widget_get_visible (GTK_WIDGET (priv->chapters_view));
-#else
-  return GTK_WIDGET_VISIBLE (priv->chapters_view);
-#endif
 }
 
 void
@@ -1581,11 +1657,7 @@ gucharmap_charmap_get_page_visible (GucharmapCharmap *charmap,
   if (!page_widget)
     return FALSE;
 
-#if GTK_CHECK_VERSION (2,18,0)
   return gtk_widget_get_visible (page_widget);
-#else
-  return GTK_WIDGET_VISIBLE (page_widget);
-#endif
 }
 
 void
@@ -1605,6 +1677,15 @@ gucharmap_charmap_get_active_page (GucharmapCharmap *charmap)
   return priv->active_page;
 }
 
+/**
+ * gucharmap_charmap_get_active_codepoint_list:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Gets the @GucharmapCodepointList associated with the #GucharmapChartable of
+ * @charmap.
+ *
+ * Returns: (transfer none): the #GucharmapCodepointList
+ */
 GucharmapCodepointList *
 gucharmap_charmap_get_active_codepoint_list (GucharmapCharmap *charmap)
 {
